(function() {
    'use strict';

    var storage;

    try {
        var backgroundPage = chrome.extension.getBackgroundPage();
        panels(backgroundPage);
    } catch(Exception) {
        chrome.runtime.getBackgroundPage(panels);
    }

    // very simple cache
    window.cache = {};

    function panels(backgroundPage) {
        storage = backgroundPage.storage;

        $(function() {
            // show/hide the login form
            function showLoginForm() {
                $('.login-section').removeClass('hide');

                // reset the login form
                $('.login-form :input:visible').val('');
                resetButton();

                // focus the first input field
                $('.login-form :input:visible:first').focus();
            }
            function hideLoginForm() {
                $('.login-section').addClass('hide');
            }

            // show/hide the panel's content
            function showPanel() {
                $('.container').removeClass('hide');
            }
            function hidePanel() {
                $('.container').addClass('hide');
            }

            // capture keys in login form
            $('.login-form :input').keydown(function(e) {
                switch(e.which) {
                    // cycle through proper fields with tab
                    case 9:
                        var that = this;
                        var inputs = $('.login-form :input').filter(function(index, input) {
                            return that.tabIndex < input.tabIndex;
                        });

                        if(inputs.length === 0) {
                            $('#username').focus();
                        } else {
                            $(inputs[0]).focus();
                        }

                        e.preventDefault();
                        break;
                    // login on enter
                    case 13:
                        login();

                        e.preventDefault();
                        break;
                }

                if($('.login-button').hasClass('temporary-text')) {
                    resetButton();
                }
            });

            // attempt to login
            function login() {
                // login when form is not empty
                if($('#username').val().trim().length && $('#password').val().length) {
                    chrome.runtime.sendMessage({'login.attempt': {
                        username: $('#username').val().trim(),
                        password: $('#password').val()
                    }});
                }
            }

            // login with the button
            $('.login-button').click(function() {
                if($('.login-button').hasClass('temporary-text')) {
                    resetButton();
                } else {
                    login();
                }
            });

            // when logging in, display an indicator
            function busyLoginButton() {
                var button = $('.login-button');
                $(button).data('reset-text', $(button).html());
                $(button)
                    .html($(button).data('loading-text'))
                    .prop('disabled', true)
                    .addClass('loading');
            }
            chrome.runtime.onMessage.addListener(
                function(request, sender, sendResponse) {
                    if(request == 'login.indicator.start') {
                        busyLoginButton();
                    }
                });

            // reset the login indicator
            function resetButton() {
                var button = $('.login-button');
                $(button)
                    .html($(button).data('reset-text'))
                    .prop('disabled', false)
                    .removeClass('loading')
                    .removeClass('failed')
                    .removeClass('info')
                    .removeClass('temporary-text');
            }
            chrome.runtime.onMessage.addListener(
                function(request, sender, sendResponse) {
                    if(request == 'login.indicator.stop') {
                        resetButton();
                    }
                });

            // show an error on login fail
            function failedLoginButton() {
                var button = $('.login-button');
                $(button)
                    .html($(button).data('failed-text'))
                    .prop('disabled', false)
                    .addClass('failed')
                    .addClass('temporary-text');
            }
            chrome.runtime.onMessage.addListener(
                function(request, sender, sendResponse) {
                    if(request == 'login.failed') {
                        failedLoginButton();
                    }
                });

            // show a message on logout
            function loggedOutButton() {
                var button = $('.login-button');
                $(button)
                    .html($(button).data('logout-text'))
                    .prop('disabled', false)
                    .addClass('info')
                    .addClass('temporary-text');
            }
            chrome.runtime.onMessage.addListener(
                function(request, sender, sendResponse) {
                    if(request == 'logout') {
                        hidePanel();
                        showLoginForm();

                        loggedOutButton();
                    }
                });

            // after login, show the user's email address
            chrome.runtime.onMessage.addListener(
                function(request, sender, sendResponse) {
                    if(request.hasOwnProperty('login.success')) {
                        var user = request['login.success'].user;
                        $('#user-name').text(user.email);

                        hideLoginForm();
                        showPanel();
                    }
                });

            // capture icon clicks in the plugin container
            $('#logout').click(function(event) {
                chrome.runtime.sendMessage('logout.attempt');
            });
            $('#help').click(function(event) {
                chrome.runtime.sendMessage('help');
                // chrome.runtime.sendMessage('close');
                window.close();
            });
            $('#settings').click(function(event) {
                chrome.runtime.sendMessage('settings');
                // chrome.runtime.sendMessage('close');
                window.close();
            });
            $('#close').click(function() {
                chrome.runtime.sendMessage('close');
                window.close();
            });

            // keep track whether this popup is open or closed
            storage.put('isMainPanelOpen', true);
            $(window).unload(function() {
                storage.put('isMainPanelOpen', false);
            });

            /**
             * default_popup (panel.html) is reloaded every time, so the only way to 'persist' the data
             * is by reading data from storage and present them as they were.
             */
            (function restore() {
                if(storage.get('user') && storage.get('username') && storage.get('password')) {
                    chrome.runtime.sendMessage('restore');

                    var user = storage.get('user');
                    $('#user-name').text(user.email);

                    hideLoginForm();
                    showPanel();
                } else {
                    console.info('no saved state');
                }
            })();
        });
    }
})();
